@@ -27,13 +27,17 @@ class Controls:
     This is the main class for the optimization quantities, i.e. pulses, parameters, and times.
     All these quantities are defined in this class and can be accessed by calling the modules here.
     """
-    def __init__(self, pulses_list, times_list, parameters_list, rng: RandomNumberGenerator = None):
+
+    def __init__(self, pulses_list, times_list, parameters_list, rng: RandomNumberGenerator = None,
+                 is_AD: bool = False):
         """
         Constructor of the general class containing all the controls used during the optimization
 
         :param pulses_list: List containing the settings for each pulse
         :param times_list: List containing the settings for each time
         :param parameters_list: List containing the setting for each parameter
+        :param rng: Random number generator object (optional)
+        :param is_AD: Boolean flag to activate the automatic differentiation (optional)
         """
         # Map index
         map_index = -1
@@ -54,7 +58,7 @@ class Controls:
                                              name=basis.setdefault("basis_name", None),
                                              class_type='basis')
             # Create the pulse obj
-            self.pulse_objs_list.append(basis_attribute(map_index, pulse, rng=rng))
+            self.pulse_objs_list.append(basis_attribute(map_index, pulse, rng=rng, is_AD=is_AD))
             # Update the map index for the next control
             map_index = self.pulse_objs_list[-1].last_index
         ###############################################
@@ -73,8 +77,33 @@ class Controls:
             self.times_obj_dictionary[time["time_name"]] = TimeParameter(**time)
             # TODO Implement the time optimization here
 
+        # Custom get control functions for AD
+        if is_AD:
+            self.get_controls_lists = self._get_controls_jax_obj
+            self.get_final_controls_lists = self._get_converted_jax_obj
+        else:
+            self.get_controls_lists = self._get_controls_lists
+            self.get_final_controls_lists = self._get_controls_lists
+        # Calculate the biggest bins number among the different pulses
+        if len(self.pulse_objs_list) > 0:
+            self.max_bin_numbers = np.max(np.asarray([pulse.bins_number for pulse in self.pulse_objs_list], dtype=int))
+        else:
+            self.max_bin_numbers = 0
+        # Load the AD functions
+        if is_AD:
+            self._buffer_AD_arrays()
+
+    def _buffer_AD_arrays(self):
+        """
+        Imports jax.numpy if AD is activated
+        """
+        import jax.numpy as jnp
+        import jax
+        # self.debug_print = jax.debug.print
+        self.jnp = jnp
+
     def get_control_parameters_number(self) -> int:
-        """Return the control parameter number"""
+        """Returns the number of control parameters"""
         ###############################################
         # Pulses
         ###############################################
@@ -96,16 +125,17 @@ class Controls:
         return control_parameters_number
 
     def select_basis(self) -> None:
-        """Initialize the superparameter basis"""
+        """Initializes the super parameter basis"""
         for pulse in self.pulse_objs_list:
-            pulse.super_parameter_distribution_obj.set_random_super_parameter()
-            # Update the base pulse parameters and functions
-            pulse.update_chopped_basis()
-            # Update the control parameter indexes
-            self._update_control_parameter_indexes()
+            if isinstance(pulse, ChoppedBasis):
+                pulse.super_parameter_distribution_obj.set_random_super_parameter()
+                # Update the base pulse parameters and functions
+                pulse.update_chopped_basis()
+                # Update the control parameter indexes
+                self._update_control_parameter_indexes()
 
     def _update_control_parameter_indexes(self) -> None:
-        """Update the control parameter indexes"""
+        """Updates the control parameter indexes"""
         # Map index
         map_index = -1
         ###############################################
@@ -129,7 +159,11 @@ class Controls:
         # Return the control parameters number
 
     def get_random_super_parameter(self) -> np.array:
-        """Return list with dcrab current super_parameters"""
+        """
+        Returns a list with current super_parameters
+
+        :return np.array: List with current super_parameters
+        """
         super_parameter_list = []
         for pulse in self.pulse_objs_list:
             if isinstance(pulse, ChoppedBasis):
@@ -138,9 +172,10 @@ class Controls:
         return super_parameter_array
 
     def get_sigma_variation(self) -> np.array:
-        """Return a numpy array with the maximum sigma in the parameters choice for the start simplex
+        """
+        Returns a numpy array with the maximum sigma in the parameters choice for the start simplex
 
-        :return np.array
+        :return np.array:
         """
         sigma_variation_coefficients = np.zeros(self.get_control_parameters_number(), dtype="float")
         # Pulses
@@ -157,7 +192,8 @@ class Controls:
         return sigma_variation_coefficients
 
     def get_mean_value(self) -> np.array:
-        """Return a numpy array the mean value
+        """
+        Return a numpy array with the mean value of the control parameters
 
         :return np.array:
         """
@@ -173,7 +209,11 @@ class Controls:
         return mean_value_coefficients
 
     def update_base_controls(self, optimized_parameters_vector: np.array) -> None:
-        """Update the base controls. Only pulses is enough"""
+        """
+        Updates the base controls with the optimized parameters.
+
+        :param np.array optimized_parameters_vector: Optimized parameters vector
+        """
         # Set the times
         for time_name in self.times_obj_dictionary:
             time = self.times_obj_dictionary[time_name]
@@ -188,9 +228,9 @@ class Controls:
         for parameter in self.parameter_objs_list:
             parameter.set_parameter(optimized_parameters_vector[parameter.control_parameters_list])
 
-    def get_controls_lists(self, optimized_parameters_vector: np.array) -> [list, list, list]:
+    def _get_controls_lists(self, optimized_parameters_vector: np.array) -> [list, list, list]:
         """
-        Set the optimized control parameters and get the controls
+        Sets the optimized control parameters and get the controls
 
         :param np.array optimized_parameters_vector:
         :return: The pulses, time grids, and the parameters in three different lists of numpy arrays.
@@ -217,12 +257,70 @@ class Controls:
 
         return pulses_list, time_grids_list, parameters_list
 
-    def get_bare_controls_lists(self, optimized_parameters_vector: np.array) -> [list]:
+    def _get_converted_jax_obj(self, optimized_parameters_vector: np.array) -> np.array:
         """
-        Set the optimized control parameters and get the controls
+        Converts the jax controls into numpy controls casting to real
 
         :param np.array optimized_parameters_vector:
-        :return: The bare OC pulses (without guess and shaping) ina list of numpy arrays.
+        :return: The pulses, time grids, and the parameters in three different lists of numpy arrays.
+        """
+        pulses_array = self.jnp.zeros((len(self.pulse_objs_list), self.max_bin_numbers))
+        time_grids_array = self.jnp.zeros((len(self.pulse_objs_list), self.max_bin_numbers))
+        parameters_array = self.jnp.zeros((len(self.parameter_objs_list),))
+        # Set the times
+        for time_name in self.times_obj_dictionary:
+            time_obj = self.times_obj_dictionary[time_name]
+            if time_obj.is_optimization:
+                time_obj.set_parameter(optimized_parameters_vector[time_obj.control_parameters_list])
+        # Get the pulses and the timegrids
+        for index, pulse in enumerate(self.pulse_objs_list):
+            time_name = pulse.time_name
+            pulse_array = pulse.get_pulse(
+                optimized_parameters_vector[np.asarray(pulse.control_parameters_list)],
+                final_time=self.times_obj_dictionary[time_name].get_time())
+            pulses_array = pulses_array.at[index, :pulse.bins_number].set(self.jnp.asarray(pulse_array))
+            time_grids_array = time_grids_array.at[index, :pulse.bins_number].set(self.jnp.asarray(pulse.time_grid))
+        # Get the parameters
+        for index, parameter in enumerate(self.parameter_objs_list):
+            parameters_array[index] = parameter.get_parameter(
+                optimized_parameters_vector[parameter.control_parameters_list])
+        return pulses_array, time_grids_array, parameters_array
+
+    def _get_controls_jax_obj(self, optimized_parameters_vector: np.array) -> np.array:
+        """
+        Sets the optimized control parameters and gets the controls in case of jax (AD) optimization
+
+        :param np.array optimized_parameters_vector:
+        :return: The pulses, time grids, and the parameters in three different lists of jax.numpy arrays.
+        """
+        pulses_array = self.jnp.zeros((len(self.pulse_objs_list), self.max_bin_numbers), dtype=self.jnp.complex64)
+        time_grids_array = self.jnp.zeros((len(self.pulse_objs_list), self.max_bin_numbers), dtype=self.jnp.complex64)
+        parameters_array = self.jnp.zeros((len(self.parameter_objs_list),), dtype=self.jnp.complex64)
+        # Set the times
+        for time_name in self.times_obj_dictionary:
+            time_obj = self.times_obj_dictionary[time_name]
+            if time_obj.is_optimization:
+                time_obj.set_parameter(optimized_parameters_vector[time_obj.control_parameters_list])
+        # Get the pulses and the timegrids
+        for index, pulse in enumerate(self.pulse_objs_list):
+            time_name = pulse.time_name
+            pulse_array = pulse.get_pulse(
+                optimized_parameters_vector[np.asarray(pulse.control_parameters_list)],
+                final_time=self.times_obj_dictionary[time_name].get_time())
+            pulses_array = pulses_array.at[index, :pulse.bins_number].set(self.jnp.asarray(pulse_array, dtype=self.jnp.complex64))
+            time_grids_array = time_grids_array.at[index, :pulse.bins_number].set(self.jnp.asarray(pulse.time_grid, dtype=self.jnp.complex64))
+        # Get the parameters
+        for index, parameter in enumerate(self.parameter_objs_list):
+            parameters_array[index] = parameter.get_parameter(
+                optimized_parameters_vector[parameter.control_parameters_list])
+        return pulses_array, time_grids_array, parameters_array
+
+    def get_bare_controls_lists(self, optimized_parameters_vector: np.array) -> [list]:
+        """
+        Sets the optimized control parameters and gets the bare controls (without guess and shaping)
+
+        :param np.array optimized_parameters_vector:
+        :return: The bare OC pulses (without guess and shaping) in a list of numpy arrays.
         """
         bare_OC_pulses_list = []
 
