@@ -33,7 +33,8 @@ class AllInOneCommunication:
                  dump_attribute: callable = DummyDump,
                  comm_signals_list: [list, list, list] = None,
                  create_logfile: bool = True,
-                 dump_format: str = "npz"):
+                 dump_format: str = "npz",
+                 optimization_direction: str = "minimization"):
         """
         In case the user chooses to run the optimization in his device, this class is used by the OptimizationAlgorithm.
         The objects to dump the results, calculate the figure of merit, and the logger are created here. 
@@ -85,6 +86,10 @@ class AllInOneCommunication:
         self.controls_dict = {}
         # Initialize the controls names dictionary
         self.controls_names_dict = {}
+        # multiply by -1 if the optimization is a maximization
+        self.FoM_factor = 1
+        if optimization_direction == "maximization":
+            self.FoM_factor = -1
 
     def print_logger(self, message: str = "", level: int = 20):
         """Print a message in the log"""
@@ -118,23 +123,14 @@ class AllInOneCommunication:
         self.logger.info("Setting is_running state: {0}".format(self.he_obj.is_user_running))
 
     def send_controls(self, controls_dict: dict) -> None:
-        """
-        Set the controls for FoM calculation and notify the gui
-
-        :param dict controls_dict:
-        :return:
-        """
+        """Set the controls for FoM calculation and notify the gui"""
         self.controls_dict = controls_dict
         if self.controls_update_signal is not None:
             self.controls_update_signal.emit(controls_dict["pulses"], controls_dict["timegrids"],
                                              controls_dict["parameters"])
 
     def set_controls_names(self, controls_names_dict: dict) -> None:
-        """
-         Set the controls names dictionary
-         :param dict controls_names_dict:
-         :return:
-         """
+        """Set the controls names dictionary"""
         self.controls_names_dict = controls_names_dict
 
     def get_data(self) -> dict:
@@ -145,6 +141,8 @@ class AllInOneCommunication:
         """
         self.logger.debug("User running: {0}".format(self.he_obj.is_user_running))
         FoM_dict = self.FoM_obj.get_FoM(**self.controls_dict)
+        # multiply by -1 if the optimization is a maximization
+        FoM_dict["FoM"] *= self.FoM_factor
         # set the status of the FoM to 0 if it does not exist already
         status_code = FoM_dict.setdefault("status_code", 0)
         # if the user passes a different status code than zero, stop the optimization
@@ -176,7 +174,7 @@ class AllInOneCommunication:
     def _print_general_log(self, iteration_number: int, FoM: float):
         """Print the general log at each function evaluation"""
         if self.print_general_log:
-            self.logger.info("Function evaluation number: {0}, FoM: {1}".format(iteration_number, FoM))
+            self.logger.info("Function evaluation number: {0}, FoM: {1}".format(iteration_number, self.FoM_factor*FoM))
 
     def update_controls(self, **response_for_client) -> None:
         """External call to update the controls"""
