@@ -14,13 +14,33 @@
 #  limitations under the License.
 # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
-from quocslib.optimalcontrolproblems.su2 import hamiltonian_d1_d2_2fields
 import numpy as np
 from scipy.linalg import expm, norm
 from quocslib.utils.AbstractFoM import AbstractFoM
 
 
+def hamiltonian_d1_d2_2fields(amplitude_t, phase_t, delta1=0.0, delta2=0.0):
+    """
+    The Hamiltonian to use for the OneQubit problem with 2 driving fields.
+    :param amplitude_t: amplitude of the driving field
+    :param phase_t: phase between sigma_x and sigma_y
+    :param delta1: detuning on the energy levels
+    :param delta2: detuning on the drive
+    :return: The Hamiltonian
+    """
+    sigma_x = np.array([[0, 1], [1, 0]], dtype="complex")
+    sigma_y = np.array([[0, -1j], [1j, 0]], dtype="complex")
+    sigma_z = np.array([[1, 0], [0, -1]], dtype="complex")
+
+    ham_t = delta1 * sigma_z / 2 + amplitude_t * (1 + delta2) * (np.cos(phase_t) * sigma_x + np.sin(phase_t) * sigma_y)
+    return ham_t
+
+
 class OneQubit2Fields(AbstractFoM):
+    """
+    This class implements the one qubit problem with two time-dependent function in the drive
+    as an example FoM class.
+    """
     def __init__(self, args_dict: dict = None):
         if args_dict is None:
             args_dict = {}
@@ -30,13 +50,23 @@ class OneQubit2Fields(AbstractFoM):
             dtype="complex",
         )
         self.psi_0 = np.asarray(eval(args_dict.setdefault("initial_state", "[1.0, 0.0]")), dtype="complex")
+
+        # two constant detuning values to use in the Hamiltonian
         self.delta1 = args_dict.setdefault("delta1", 0.1)
         self.delta2 = args_dict.setdefault("delta2", 0.1)
+
         # Noise in the figure of merit
         self.is_noisy = args_dict.setdefault("is_noisy", False)
         self.noise_factor = args_dict.setdefault("noise_factor", 0.05)
 
     def get_FoM(self, pulses: list = [], parameters: list = [], timegrids: list = []) -> dict:
+        """
+        This function calculates the figure of merit for the one qubit problem with two fields.
+        :param pulses:
+        :param parameters:
+        :param timegrids:
+        :return dict: The FoM and the standard deviation in a dictionary
+        """
         amplitude = np.asarray(pulses[0])
         phase = np.asarray(pulses[1])
         timegrid = np.asarray(timegrids[0])
@@ -55,18 +85,31 @@ class OneQubit2Fields(AbstractFoM):
 
     @staticmethod
     def _time_evolution(amplitude, phase, dt, delta1=0.0, delta2=0.0):
+        """
+        This function calculates the time evolution operator for the one qubit problem with two fields.
+        :param amplitude:
+        :param phase:
+        :param dt:
+        :param delta1:
+        :param delta2:
+        :return:
+        """
         U = np.identity(2)
-        for ii in range(amplitude.size - 1):
-            ham_t = hamiltonian_d1_d2_2fields(
-                (amplitude[ii + 1] + amplitude[ii]) / 2,
-                (phase[ii + 1] + phase[ii]) / 2,
-                delta1,
-                delta2,
-            )
+        for ii in range(amplitude.size):
+            ham_t = hamiltonian_d1_d2_2fields(amplitude[ii],
+                                              phase[ii],
+                                              delta1,
+                                              delta2)
             U_temp = U
             U = np.matmul(expm(-1j * ham_t * dt), U_temp)
         return U
 
     @staticmethod
     def _get_fidelity(psi1, psi2):
+        """
+        This function calculates the fidelity between two states.
+        :param psi1:
+        :param psi2:
+        :return float: Fidelity between psi1 and psi2
+        """
         return np.abs(np.dot(psi1.conj().T, psi2))**2 / (norm(psi1) * norm(psi2))
