@@ -25,10 +25,23 @@ from quocslib.utils.dynamicimport import dynamic_import
 
 
 class dCRABAlgorithm(OptimizationAlgorithm):
+    """
+    This is the implementation of the dCRAB algorithm.
+    The important function are:
+    * the constructor with the optimization dictionary and the communication object as parameters
+    * run : The main loop for optimal control
+    * _get_response_for_client : return info about the goodness of the controls and errors if any
+    * _get_controls : return the set of controls as a dictionary with pulses, parameters, and times as keys
+    * _get_final_results: return the final result of the optimization algorithm
+    """
     def __init__(self, optimization_dict: dict = None, communication_obj=None, **kwargs):
         """
         This is the implementation of the dCRAB algorithm. All the arguments in the constructor are passed to the
-        OptimizationAlgorithm class except the optimization dictionary where the dCRAB settings and the controls are defined.
+        OptimizationAlgorithm class except the optimization dictionary where the dCRAB settings and the controls are
+        defined.
+
+        :param optimization_dict: dictionary with the optimization settings and the controls
+        :param communication_obj: object to communicate with the client
         """
         super().__init__(communication_obj=communication_obj, optimization_dict=optimization_dict)
         ###########################################################################################
@@ -82,7 +95,7 @@ class dCRABAlgorithm(OptimizationAlgorithm):
             if "re_evaluation_steps" in re_evaluation_parameters:
                 self.re_evaluation_steps = np.asarray(re_evaluation_parameters["re_evaluation_steps"], dtype=float)
             else:
-                self.re_evaluation_steps = np.asarray([0.3, 0.5, 0.6], dtype=float)
+                self.re_evaluation_steps = np.asarray([0.3, 0.5, 0.501], dtype=float)
                 message = "Steps not found. The default will be used in the optimization: {0}".format(
                     self.re_evaluation_steps)
                 self.comm_obj.print_logger(message, level=30)
@@ -129,7 +142,12 @@ class dCRABAlgorithm(OptimizationAlgorithm):
         self.iteration_number_list: list = []
 
     def _get_response_for_client(self) -> dict:
-        """ Return useful information for the client interface and print message in the log """
+        """
+        This function returns the response for the client
+
+        :return dict: Information dictionary with the FoM, iteration number, SI number, status code, and if the FoM
+        is a new record
+        """
         # Get the average FoM
         FoM, std = self._get_average_FoM_std()
         status_code = self.FoM_dict.setdefault("status_code", 0)
@@ -137,7 +155,7 @@ class dCRABAlgorithm(OptimizationAlgorithm):
         if self.re_evaluation_steps is None:
             if self.get_is_record(FoM):
                 message = "New record achieved. Previous FoM: {FoM}, new best FoM : {best_FoM}".format(
-                    FoM=self.best_FoM, best_FoM=FoM)
+                    FoM=self.FoM_factor*self.best_FoM, best_FoM=self.FoM_factor*FoM)
                 self.comm_obj.print_logger(message=message, level=20)
                 self.best_FoM = FoM
                 self.best_xx = self.xx.copy()
@@ -160,18 +178,18 @@ class dCRABAlgorithm(OptimizationAlgorithm):
                                                            sub_it=self.alg_iteration_number))
         # Data
         if self.re_evaluation_steps is not None:
-            message += ", Re-eval. number: {0}, FoM: {1}, std: {2}".format(self.step_number, FoM, std)
+            message += ", Re-eval. number: {0}, FoM: {1}, std: {2}".format(self.step_number, self.FoM_factor*FoM, std)
         else:
-            message += ", FoM: {0}".format(FoM)
+            message += ", FoM: {0}".format(self.FoM_factor*FoM)
         self.comm_obj.print_logger(message, level=20)
         # Load the current figure of merit and iteration number in the summary list of dCRAB
         if status_code == 0:
-            self.FoM_list.append(FoM)
+            self.FoM_list.append(self.FoM_factor*FoM)
             self.iteration_number_list.append(self.iteration_number)
         return response_dict
 
     def run(self) -> None:
-        """Main loop of the dCRAB method"""
+        """Starts the main loop of the optimization"""
         for super_it in range(1, self.max_num_si + 1):
             # Check if the optimization was stopped by the user
             if not self.is_optimization_running():
@@ -211,14 +229,17 @@ class dCRABAlgorithm(OptimizationAlgorithm):
         x0 = self.controls.get_mean_value()
         # Evaluate the FoM with the standard routine call and set the FoM as the current record
         iteration, self.is_record, self.step_number = 0, True, 0
-        self.best_FoM = self._routine_call(x0, iteration)
+        new_FoM = 0
+        for _ in range(self.compensate_drift_num_average):
+            new_FoM += self._routine_call(x0, iteration)
+        self.best_FoM = new_FoM / self.compensate_drift_num_average
         # Info message
-        message = f"Previous best FoM: {previous_best_FoM}, Current best FoM after drift " \
-                  f"compensation ({mode}): {self.best_FoM}"
+        message = f"Previous best FoM: {self.FoM_factor*previous_best_FoM}, Current best FoM after drift " \
+                  f"compensation ({mode}): {self.FoM_factor*self.best_FoM}"
         self.comm_obj.print_logger(message=message, level=20)
         # At this point is not necessary to set again is_record to False since is newly re-define at the beginning of
         # the _inner_routine_call function
-        # TODO: Thinks if makes sense to update the sigma best value here
+        # TODO: Think about if it makes sense to update the sigma best value here
 
     def _update_base_pulses(self) -> None:
         """Update the base dCRAB pulse with the best controls found so far"""
@@ -251,102 +272,116 @@ class dCRABAlgorithm(OptimizationAlgorithm):
                    "Best FoM: {best_FoM}, Terminate reason: {reason}\n".format(super_it=self.super_it,
                                                                                NfunevalsUsed=NfunevalsUsed,
                                                                                termination_reason=self.terminate_reason,
-                                                                               best_FoM=self.best_FoM,
+                                                                               best_FoM=self.FoM_factor*self.best_FoM,
                                                                                reason=self.terminate_reason))
         self.comm_obj.print_logger(message=message, level=20)
 
     def _inner_routine_call(self, optimized_control_parameters: np.array, iterations: int,
-                            drift_comp=False, drift_comp_average=1) -> float:
-        """This is an inner method for function evaluation. It is useful when the user wants to evaluate the FoM
-        with the same controls multiple times to take into accounts noise in the system"""
-        # set the is_record to False unless we have drift_comp call... then we want to update the current best FoM
-        self.is_record = False
-        # Initialize step number to 0
-        self.step_number = 0
-        FoM = -1.0 * self.optimization_factor * self._routine_call(optimized_control_parameters, iterations)
-        # This has to be here because it takes the hard-coded initial FoM of 1e10 if the optimization is stopped
-        # before the first search iteration has finished
-        if self.optimization_direction == "maximization" and FoM == 1e10:
-            FoM = -FoM
-        ################################################################################################################
-        # Standard function evaluation - dCRAB without re-evaluation steps
-        ################################################################################################################
-        if self.re_evaluation_steps is None:
-            mu_1 = FoM
-        else:
-            ############################################################################################################
-            # Implement the re-evaluation step method
-            ############################################################################################################
-            # check mu-sig criterion by calculating probability of current pulses being new record
-            # Re evaluation steps initialization e.g. [0.33, 0.5, 0.501, 0.51]
-            re_evaluation_steps = self.re_evaluation_steps
-            # First evaluation in whole optimization -> do not reevaluate
-            if self.iteration_number == 1:
-                re_evaluation_steps = np.array([0.5])
-            # number of steps
-            max_steps_number = re_evaluation_steps.shape[0]
-            # Initialize to zero the FoM_test and the sigma_test arrays
-            self.FoM_test = 0.0 * self.FoM_test
-            self.sigma_test = 0.0 * self.sigma_test
-            # Get the figure of merit from the client
-            self.FoM_test[0] = FoM
-            # TODO: Check if optimization_is_running is necessary here
-            # Get the standard deviation
-            self.sigma_test[0] = float(self.FoM_dict.setdefault("std", 1.0))
-            # Increase step number after function evaluation
-            self.step_number += 1
-            # p level test better than current record
-            for ii in range(max_steps_number):
-                p_level = re_evaluation_steps[ii]
-                mu_1, sigma_1 = self._get_average_FoM_std(mu_sum=np.sum(self.FoM_test) * 1.0,
-                                                          sigma_sum=np.sum(self.sigma_test) * 1.0)
-                mu_2, sigma_2 = self.best_FoM, self.best_sigma
-                probability = self._probabnormx1betterx2(mu_1, sigma_1, mu_2, sigma_2)
-                # If probability is lower than the probability in the list return the
-                if probability < p_level:
-                    return mu_1
-                # else: go on with further re-evaluations
-                self.FoM_test[ii + 1] = -1.0 * self.optimization_factor * self._routine_call(
-                    optimized_control_parameters, iterations)
-                self.sigma_test[ii + 1] = float(self.FoM_dict.setdefault("std", 1.0))
-                # Increase step number after function evaluation
-                self.step_number += 1
+                            drift_comp_new_val=None) -> float:
+        """
+        This is an inner method for function evaluation. It is useful when the user wants to evaluate the FoM
+        with the same controls multiple times to take into account noise in the system
 
-            # check if last threshold (re_evaluation_steps[-1]) is surpassed -> new record
-            mu_1, sigma_1 = self._get_average_FoM_std(mu_sum=np.sum(self.FoM_test) * 1.0,
-                                                      sigma_sum=np.sum(self.sigma_test) * 1.0)
-            mu_2, sigma_2 = self.best_FoM, self.best_sigma
-            probability = self._probabnormx1betterx2(mu_1, sigma_1, mu_2, sigma_2)
-            # TODO: Check what best FoM means in this case
-            if probability > re_evaluation_steps[-1]:
-                # We have a new record
-                self.best_sigma, self.best_FoM = sigma_1, mu_1
-                self.is_record = True
-                message = "New record achieved. New best FoM: {0}, std: {1}".format(mu_1, sigma_1)
-                self.comm_obj.print_logger(message, level=20)
-                self.best_xx = self.xx.copy()
-                self.comm_obj.update_controls(is_record=True,
-                                              FoM=self.best_FoM,
-                                              sigma=self.best_sigma,
-                                              super_it=self.super_it,
-                                              iteration_number=self.iteration_number)
-
-        if drift_comp:
+        :param optimized_control_parameters: optimized control parameters
+        :param iterations: number of iterations performed so far
+        :param drift_comp_new_val: new value of the FoM due to drift compensation
+        """
+        if drift_comp_new_val is not None:
+            mu_1 = drift_comp_new_val
             self.best_FoM = mu_1
             self.is_record = True
-            message = "New record due to drift compensation. New best FoM: {0}".format(mu_1)
+            message = "New record due to drift compensation. New best FoM: {0}".format(self.FoM_factor*mu_1)
             self.comm_obj.print_logger(message, level=20)
             self.best_xx = self.xx.copy()
             self.comm_obj.update_controls(is_record=True,
                                           FoM=self.best_FoM,
                                           super_it=self.super_it,
                                           iteration_number=self.iteration_number)
+        else:
+            # set the is_record to False unless we have drift_comp call... then we want to update the current best FoM
+            self.is_record = False
+            # Initialize step number to 0
+            self.step_number = 0
+            FoM = self._routine_call(optimized_control_parameters, iterations)
+            ###########################################################################################################
+            # # ToDo: Check if this needs to be here or not
+            # # This has to be here because it takes the hard-coded initial FoM of 1e10 if the optimization is stopped
+            # # before the first search iteration has finished
+            # if self.optimization_direction == "maximization" and FoM == 1e10:
+            #     FoM = -FoM
+            ###########################################################################################################
+            # Standard function evaluation - dCRAB without re-evaluation steps
+            ###########################################################################################################
+            if self.re_evaluation_steps is None:
+                mu_1 = FoM
+            else:
+                #######################################################################################################
+                # Implement the re-evaluation step method
+                #######################################################################################################
+                # check mu-sig criterion by calculating probability of current pulses being new record
+                # Re evaluation steps initialization e.g. [0.33, 0.5, 0.501, 0.51]
+                re_evaluation_steps = self.re_evaluation_steps
+                # First evaluation in whole optimization -> do not reevaluate
+                if self.iteration_number == 1:
+                    re_evaluation_steps = np.array([0.5])
+                # number of steps
+                max_steps_number = re_evaluation_steps.shape[0]
+                # Initialize to zero the FoM_test and the sigma_test arrays
+                self.FoM_test = 0.0 * self.FoM_test
+                self.sigma_test = 0.0 * self.sigma_test
+                # Get the figure of merit from the client
+                self.FoM_test[0] = FoM
+                # TODO: Check if optimization_is_running is necessary here
+                # Get the standard deviation
+                self.sigma_test[0] = float(self.FoM_dict.setdefault("std", 1.0))
+                # Increase step number after function evaluation
+                self.step_number += 1
+                # p level test better than current record
+                for ii in range(max_steps_number):
+                    p_level = re_evaluation_steps[ii]
+                    mu_1, sigma_1 = self._get_average_FoM_std(mu_sum=np.sum(self.FoM_test) * 1.0,
+                                                              sigma_sum=np.sum(self.sigma_test) * 1.0)
+                    mu_2, sigma_2 = self.best_FoM, self.best_sigma
+                    probability = self._probabnormx1betterx2(mu_1, sigma_1, mu_2, sigma_2)
+                    # If probability is lower than the probability in the list return the
+                    if probability < p_level:
+                        return mu_1
+                    # else: go on with further re-evaluations
+                    self.FoM_test[ii + 1] = self._routine_call(optimized_control_parameters, iterations)
+                    self.sigma_test[ii + 1] = float(self.FoM_dict.setdefault("std", 1.0))
+                    # Increase step number after function evaluation
+                    self.step_number += 1
+
+                # check if last threshold (re_evaluation_steps[-1]) is surpassed -> new record
+                mu_1, sigma_1 = self._get_average_FoM_std(mu_sum=np.sum(self.FoM_test) * 1.0,
+                                                          sigma_sum=np.sum(self.sigma_test) * 1.0)
+                mu_2, sigma_2 = self.best_FoM, self.best_sigma
+                probability = self._probabnormx1betterx2(mu_1, sigma_1, mu_2, sigma_2)
+                # TODO: Check what best FoM means in this case
+                if probability > re_evaluation_steps[-1]:
+                    # We have a new record
+                    self.best_sigma, self.best_FoM = sigma_1, mu_1
+                    self.is_record = True
+                    message = "New record achieved. New best FoM: {0}, std: {1}".format(self.FoM_factor*mu_1, sigma_1)
+                    self.comm_obj.print_logger(message, level=20)
+                    self.best_xx = self.xx.copy()
+                    self.comm_obj.update_controls(is_record=True,
+                                                  FoM=self.best_FoM,
+                                                  sigma=self.best_sigma,
+                                                  super_it=self.super_it,
+                                                  iteration_number=self.iteration_number)
 
         # Return the figure of merit to be minimized by the updating algorithm
-        return -1.0 * self.optimization_factor * mu_1
+        return mu_1
 
     def _get_average_FoM_std(self, mu_sum: float = None, sigma_sum: float = None) -> np.array:
-        """Calculate the average figure of merit and sigma"""
+        """
+        Calculate the average figure of merit and standard deviaiton
+
+        :param mu_sum: Sum of the figure of merit values for the re-evaluation steps
+        :param sigma_sum: Sum of the standard deviation values for the re-evaluation steps
+        :return: Average figure of merit and standard deviation
+        """
         step_number = self.step_number
         # For the first evaluation and in case no re-evaluation step is needed return directly
         if step_number == 0:
@@ -376,8 +411,6 @@ class dCRABAlgorithm(OptimizationAlgorithm):
         # Start by defining a new random variable z = x1 - x2
         # if mu_z > 0 the probability is > 0.5 , else: <0.5
         mu_z = mu_2 - mu_1
-        if self.optimization_direction == "maximization":
-            mu_z = mu_1 - mu_2
         std_comb = np.sqrt(sigma_1**2 + sigma_2**2)
         if np.abs(std_comb) < 10**(-14):
             # Warning message
@@ -392,7 +425,12 @@ class dCRABAlgorithm(OptimizationAlgorithm):
         return probability
 
     def _get_controls(self, xx: np.array) -> dict:
-        """Get the controls dictionary from the optimized control parameters"""
+        """
+        Get the controls dictionary from the optimized control parameters
+
+        :param xx: optimized control parameters
+        :return dict : dictionary with the controls
+        """
         [pulses, timegrids, parameters] = self.controls.get_controls_lists(xx)
 
         controls_dict = {
@@ -403,9 +441,13 @@ class dCRABAlgorithm(OptimizationAlgorithm):
         return controls_dict
 
     def _get_final_results(self) -> dict:
-        """Return a dictionary with final results to put into a dictionary"""
+        """
+        Return a dictionary with final results
+
+        :return dict: dictionary with final results
+        """
         final_dict = {
-            "Figure of merit": self.best_FoM,
+            "Figure of merit": self.FoM_factor * self.best_FoM,
             "Std": self.best_sigma,
             "total number of function evaluations": self.iteration_number,
             "dcrab_freq_list": self.dcrab_super_parameter_list,
