@@ -22,16 +22,23 @@ from quocslib.tools.randomgenerator import RandomNumberGenerator
 
 
 class DummyBasis(ChoppedBasis):
+    """
+    Dummy Basis class. It is used as a template for the creation of new basis.
+    """
     amplitude_variation: float
     optimized_control_parameters: np.ndarray
     optimized_super_parameters: np.ndarray
     time_grid: np.ndarray
 
-    def __init__(self, map_index: int, pulse_dictionary: dict, rng: RandomNumberGenerator = None):
+    def __init__(self, map_index: int, pulse_dictionary: dict, rng: RandomNumberGenerator = None, is_AD: bool = False):
         """
+        Constructor of the Dummy Basis class. It is used as a template for the creation of new basis. It calls the
+        constructor of the parent class ChoppedBasis.
 
-        :param int map_index: Index number to use to get the control parameters for the Fourier basis
-        :param dict pulse_dictionary: The dictionary of the pulse defined here. Only the basis dictionary is used btw
+        :param int map_index: Index number to use to get the control parameter.
+        :param dict pulse_dictionary: The dictionary of the pulse is defined here.
+        :param RandomNumberGenerator rng: Random number generator.
+        :param bool is_AD: Flag to indicate if the pulse is used for the automatic differentiation.
         """
         #################
         # Basis dependent settings
@@ -46,7 +53,7 @@ class DummyBasis(ChoppedBasis):
         # distribution of super parameters, etc ...
         ################
         # Constructor of the parent classes, i.e. Base Pulse and Chopped Basis
-        super().__init__(map_index=map_index, **pulse_dictionary)
+        super().__init__(map_index=map_index, rng=rng, is_AD=is_AD, **pulse_dictionary)
         #################
         # Basis dependent settings
         #################
@@ -56,7 +63,12 @@ class DummyBasis(ChoppedBasis):
         self.offset_coefficients = np.zeros((self.control_parameters_number, ))
 
     def _get_shaped_pulse(self) -> np.array:
-        """Definition of the pulse parametrization. It is called at every function evaluation to build the pulse"""
+        """
+        Definition of the pulse parametrization. It is called at every function evaluation to build the pulse and
+        return it as an array.
+
+        :return np.array: The pulse as an array.
+        """
         #################
         # Standard Basis Settings: amplitude limits, amplitude variation for the simplex,
         # distribution of super parameters, etc ...
