@@ -21,16 +21,22 @@ from quocslib.tools.randomgenerator import RandomNumberGenerator
 
 
 class Fourier(ChoppedBasis):
+    """
+    Class for the Fourier basis. It inherits from the ChoppedBasis class.
+    """
     amplitude_variation: float
     optimized_control_parameters: np.ndarray
     optimized_super_parameters: np.ndarray
     time_grid: np.ndarray
 
-    def __init__(self, map_index: int, pulse_dictionary: dict, rng: RandomNumberGenerator = None):
+    def __init__(self, map_index: int, pulse_dictionary: dict, rng: RandomNumberGenerator = None, is_AD: bool = False):
         """
+        Constructor of the Fourier basis class. It calls the constructor of the parent class ChoppedBasis.
 
-        :param int map_index: Index number to use to get the control parameters for the Fourier basis
-        :param dict pulse_dictionary: The dictionary of the pulse defined here. Only the basis dictionary is used btw
+        :param int map_index: Index number to use to get the control parameter.
+        :param dict pulse_dictionary: The dictionary of the pulse is defined here.
+        :param RandomNumberGenerator rng: The random number generator.
+        :param bool is_AD: Flag to activate the automatic differentiation.
         """
         basis_dict = pulse_dictionary["basis"]
         # Frequencies number i.e. the basis vector number in the pulse parametrization
@@ -38,13 +44,18 @@ class Fourier(ChoppedBasis):
         # Number of control parameters to be optimized
         self.control_parameters_number = 2 * self.super_parameter_number
         # Constructor of the parent class, i.e. Chopped Basis
-        super().__init__(map_index=map_index, rng=rng, **pulse_dictionary)
+        super().__init__(map_index=map_index, rng=rng, is_AD=is_AD, **pulse_dictionary)
         # Define scale and offset coefficients
-        self.scale_coefficients = (self.amplitude_variation / np.sqrt(2) * np.ones((self.control_parameters_number, )))
-        self.offset_coefficients = np.zeros((self.control_parameters_number, ))
+        self.scale_coefficients = (self.amplitude_variation / np.sqrt(2) * np.ones((self.control_parameters_number,)))
+        self.offset_coefficients = np.zeros((self.control_parameters_number,))
 
     def _get_shaped_pulse(self) -> np.array:
-        """Definition of the pulse parametrization. It is called at every function evaluation to build the pulse"""
+        """
+        Definition of the pulse parametrization. It is called at every function evaluation to build the pulse and
+        return it as an array.
+
+        :return np.array: The pulse as an array.
+        """
         # Pulse definition
         pulse = np.zeros(self.bins_number)
         # Final time definition
