@@ -20,15 +20,23 @@ import logging
 
 
 class StoppingCriteria:
+    """
+    Parent class for the stopping criteria. It contains the common attributes and methods for all the stopping
+    criteria.
+    """
+
     def __init__(self, stopping_criteria: dict = None):
         """
-        Parent class for the stopping criteria
+        Constructor of the StoppingCriteria class. Sets the values from the stopping criteria dictionary or the
+        default values if not provided.
+
+        :param dict stopping_criteria: dictionary containing the settings for the stopping criteria
         """
         self.xatol = None
         self.fatol = None
         self.terminate_reason: str = ""
         self.is_converged = False
-        self.terminate_reason = "terminate_reason is still at default"
+        self.terminate_reason = "Global Stopping Criterion reached."
         self.logger = logging.getLogger("oc_logger")
         self.max_eval = stopping_criteria.setdefault("max_eval", 10**10)
         self.time_lim = stopping_criteria.setdefault("time_lim", 10**10)
@@ -44,7 +52,7 @@ class StoppingCriteria:
     @abstractmethod
     def check_stopping_criteria(self, **kwargs) -> None:
         """
-        :return:
+        Abstract method for the stopping criteria. It must be implemented in the custom stopping criteria class.
         """
         raise ValueError("Must be implemented in the custom stopping criteria")
 
@@ -55,11 +63,11 @@ class StoppingCriteria:
 
     def check_func_eval_single_direct_search(self, func_evaluations_single_direct_search: int) -> [bool, str]:
         """
-        Check whether the maximum number of function evaluations has been exceeded
+        Checks whether the maximum number of function evaluations has been exceeded
 
         :param float func_evaluations_single_direct_search: number of current function evaluation
         :return bool is_converged: True if the stopping criterion is fulfilled
-        :return str terminate_reason: reason for the terminatiom
+        :return str terminate_reason: reason for the termination
         """
         terminate_reason = "Exceeded number of allowed function evaluations per direct search."
         is_converged = False
@@ -69,7 +77,7 @@ class StoppingCriteria:
 
     def check_simplex_criterion(self, sim: np.array) -> [bool, str]:
         """
-        Check whether the simplex has converged
+        Checks whether the simplex has converged
 
         :param np.array sim: current simplex
         :return bool is_converged: True if the stopping criterion is fulfilled
@@ -83,7 +91,7 @@ class StoppingCriteria:
 
     def check_f_size(self, fsim: np.array) -> [bool, str]:
         """
-        Check whether the FoM has converged inside this simplex
+        Checks whether the FoM has converged inside this simplex
 
         :param np.array fsim: FoM values for current simplex
         :return bool is_converged: True if the stopping criterion is fulfilled
@@ -99,13 +107,13 @@ class StoppingCriteria:
 
     def reset_direct_search_start_time(self) -> None:
         """
-        Reset the direct_search_start_time for a new SI
+        Resets the direct_search_start_time for a new SI
         """
         self.direct_search_start_time = datetime.now()
 
     def check_direct_search_time_out(self) -> [bool, str]:
         """
-        Check whether the optimization in a direct search has been running for too long.
+        Checks whether the optimization in a direct search has been running for too long.
 
         :return bool is_converged: True if the stopping criterion is fulfilled
         :return str terminate_reason: reason for the terminatiom
@@ -120,13 +128,15 @@ class StoppingCriteria:
 
     def reset_curr_FoM_track_for_new_SI(self) -> None:
         """
-        Reset the current FoM track list for a new SI
+        Resets the current FoM track list for a new SI
         """
         self.curr_FoM_track = []
 
     def add_to_FoM_track(self, FoM) -> None:
         """
-        Add new entry to current FoM track list
+        Adds new entry to current FoM track list
+
+        :param float FoM: current FoM
         """
         if self.change_based_stop["cbs_funct_evals"] > 1:
             self.curr_FoM_track.append(FoM)
@@ -139,10 +149,11 @@ class StoppingCriteria:
         cbs_funct_evals = self.change_based_stop["cbs_funct_evals"]
         cbs_change = self.change_based_stop["cbs_change"]
         if len(self.curr_FoM_track) >= cbs_funct_evals:
-            m, b = np.polyfit(range(cbs_funct_evals), self.curr_FoM_track[-cbs_funct_evals:], 1)
+            last_foms = np.asarray(self.curr_FoM_track[-cbs_funct_evals:], dtype=np.float64)
+            m, b = np.polyfit(range(cbs_funct_evals), last_foms, 1)
             current_change = abs(m * cbs_funct_evals)
             # check if the trend of changes is smaller than defined
-            if (current_change < cbs_change):
+            if current_change < cbs_change:
                 self.is_converged = True
 
     def check_advanced_stopping_criteria(self) -> None:
@@ -152,6 +163,7 @@ class StoppingCriteria:
         try:
             if self.change_based_stop["cbs_funct_evals"] > 1:
                 self._check_cbs_stopping_crit()
-        except:
+        except Exception as e:
+            print(e)
             message = "Checking change-based stop failed! cbs_funct_evals not properly defined!"
             self.logger.error(message)
