@@ -18,16 +18,17 @@ from scipy.linalg import expm
 
 
 def pw_evolution(U_store, drive, A, B, n_slices, dt):
-    """Compute the piecewise evolution of a system defined by the
+    """
+    Computes the piecewise evolution of a system defined by the
     Hamiltonian H = A + drive * B and store the result in U_store
 
-    :param List[np.matrix] U_store: the storage for all of the computed propagators
-    :param np.array drive: an array of dimension n_controls x n_slices that contains the amplitudes of the pulse
-    :param np.matrix A: the drift Hamiltonian
-    :param List[np.matrix] B: the control Hamiltonians
-    :param int n_slices: number of slices
-    :param float dt: the duration of each time slice
-    :return None: Stores the new propagators so this doesn't return
+    :param List[np.matrix] U_store: The storage for all the computed propagators
+    :param np.array drive: An array of dimension n_controls x n_slices that contains the amplitudes of the pulse
+    :param np.matrix A: The drift Hamiltonian
+    :param List[np.matrix] B: The control Hamiltonians in a list
+    :param int n_slices: Number of slices
+    :param float dt: The duration of each time slice
+    :return List[np.matrix] U_store: The computed propagators
     """
     K = len(B)
     for i in range(n_slices):
@@ -38,20 +39,21 @@ def pw_evolution(U_store, drive, A, B, n_slices, dt):
     return U_store
 
 
-def pw_final_evolution(drive, A, B, n_slices, dt, u0):
-    """Compute the piecewise evolution of a system defined by the
+def pw_final_evolution(drive, A, B, n_slices, dt, U0):
+    """
+    Computes the piecewise evolution of a system defined by the
     Hamiltonian H = A + drive * B and concatenate all the propagators
 
-    :param List[np.matrix] U_store: the storage for all of the computed propagators
-    :param np.array drive: an array of dimension n_controls x n_slices that contains the amplitudes of the pulse
-    :param np.matrix A: the drift Hamiltonian
-    :param List[np.matrix] B: the control Hamiltonians
-    :param int n_slices: number of slices
-    :param np.matrix u0: the initial density matrix to start from
-    :return np.matrix: the final propagator
+    :param np.array drive: An array of dimension n_controls x n_slices that contains the amplitudes of the pulse
+    :param np.matrix A: The drift Hamiltonian
+    :param List[np.matrix] B: The control Hamiltonians in a list
+    :param int n_slices: Number of slices
+    :param float dt: The duration of each time slice
+    :param np.matrix U0: The initial propagator to start from
+    :return np.matrix: The final propagator
     """
     K = len(B)
-    U = u0
+    U = U0
     for i in range(n_slices):
         H = A
         for k in range(K):
